==> nest-infrastructure-middleware-rdb-3.5.3.0-20251216.064212-15-sources.jar: com/digiwin/dap/nest/infrastructure/middleware/rdb/dapper/DwNodeSpanMybatis.java <==
package com.digiwin.dap.nest.infrastructure.middleware.rdb.dapper;

import com.digiwin.dap.nest.kernel.core.config.DwConfig;
import com.digiwin.dap.nest.kernel.core.dapper.aspect.JaDapperAspectPointcut;
import com.digiwin.dap.nest.kernel.core.dapper.aspect.JaNodeSpanResolver;
import com.digiwin.dap.nest.kernel.core.dapper.log.DwLog;
import com.digiwin.dap.nest.kernel.core.dapper.meta.NodeKind;
import com.digiwin.dap.nest.kernel.core.dapper.meta.NodeSpan;
import com.digiwin.dap.nest.kernel.core.util.DwStringUtil;
import com.digiwin.dap.nest.kernel.core.util.datastructure.JaCollectionUtil;
import com.digiwin.dap.nest.infrastructure.spring.util.DwSpringContextUtil;
import org.aopalliance.intercept.MethodInvocation;
import org.mybatis.spring.annotation.MapperScan;
import org.mybatis.spring.annotation.MapperScans;
import org.springframework.aop.framework.ReflectiveMethodInvocation;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.stereotype.Component;
import org.springframework.util.ClassUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author chenjian
 * @since 2024年07月08日 13:38:02
 */
@Component
@ConditionalOnClass(name = {"org.mybatis.spring.annotation.MapperScan", DwNodeSpanMybatis.ProxyClassName})
public class DwNodeSpanMybatis implements JaNodeSpanResolver, JaDapperAspectPointcut {

    public static final String ProxyClassName = "org.apache.ibatis.binding.MapperProxy";

    @Override
    public NodeSpan getNodeSpan(MethodInvocation invocation) {
        if (invocation instanceof ReflectiveMethodInvocation) {
            Object proxy = ((ReflectiveMethodInvocation) invocation).getProxy();
            if (proxy.toString().startsWith(ProxyClassName)) {
                return get(NodeKind.Constant.Db);
            }
        }
        return null;
    }

    @Override
    public String getExpression() {
        String expression = DwConfig.get("dapper.aspect.pointcut.mybatis");
        if (DwStringUtil.isNotEmpty(expression)) {
            return expression + " || @within(org.apache.ibatis.annotations.Mapper)";
        }

        List<String> basePackages = new ArrayList<>(addBasePackages(DwSpringContextUtil.getMainApplicationClass()));
//        String[] beanDefinitionNames = JaSpringContextUtil.getApplicationContext().getBeanDefinitionNames();
//        for (String beanDefinitionName : beanDefinitionNames) {
//            basePackages.addAll(addBasePackages(Objects.requireNonNull(JaSpringBeanUtil.getBean(beanDefinitionName)).getClass()));
//        }
        Set<String> basePackageSet = new HashSet<>();
        basePackages.forEach(s -> basePackageSet.addAll(Arrays.asList(s.split(","))));
        if (basePackageSet.size() > 1) {
            DwLog.get().warn("multiple mybatis mapper scan package");
        }

        StringBuilder expressionBuilder = new StringBuilder();
        for (String s : basePackageSet) {
            expressionBuilder.append(String.format("execution(* %s..*.*(..)) || ", s));
        }
        return expressionBuilder + "@within(org.apache.ibatis.annotations.Mapper)";
    }

    private List<String> addBasePackages(Class<?> importingClass) {
        List<String> basePackages = new ArrayList<>();
        MapperScans mapperScans = importingClass.getAnnotation(MapperScans.class);
        if (null != mapperScans) {
            basePackages.addAll(addBasePackages(mapperScans, importingClass));
        } else {
            MapperScan mapperScan = importingClass.getAnnotation(MapperScan.class);
            if (null != mapperScan) {
                basePackages.addAll(addBasePackages(mapperScan, importingClass));
            }
        }
        return basePackages;
    }

    private List<String> addBasePackages(MapperScans mapperScans, Class<?> importingClass) {
        List<String> basePackages = new ArrayList<>();
        if (JaCollectionUtil.isEmpty(mapperScans.value())) {
            basePackages.add(ClassUtils.getPackageName(importingClass.getName()));
        } else {
            for (MapperScan mapperScan : mapperScans.value()) {
                basePackages.addAll(addBasePackages(mapperScan, importingClass));
            }
        }
        return basePackages;
    }


    private List<String> addBasePackages(MapperScan annotation, Class<?> importingClass) {
        List<String> basePackages = new ArrayList<>();
        basePackages.addAll(
                Arrays.stream(annotation.value()).filter(StringUtils::hasText).collect(Collectors.toList()));
        basePackages.addAll(
                Arrays.stream(annotation.basePackages()).filter(StringUtils::hasText).collect(Collectors.toList()));
        basePackages.addAll(
                Arrays.stream(annotation.basePackageClasses()).map(ClassUtils::getPackageName).collect(Collectors.toList()));
        if (JaCollectionUtil.isEmpty(basePackages)) {
            basePackages.add(ClassUtils.getPackageName(importingClass));
        }
        return basePackages;
    }

}

==> nest-infrastructure-middleware-rdb-5.5.0.1000-20251217.080719-4-sources.jar: com/digiwin/dap/nest/infrastructure/middleware/rdb/dapper/DwNodeSpanMybatis.java <==
package com.digiwin.dap.nest.infrastructure.middleware.rdb.dapper;

import com.digiwin.dap.nest.kernel.core.config.DwConfig;
import com.digiwin.dap.nest.kernel.core.dapper.aspect.JaDapperAspectPointcut;
import com.digiwin.dap.nest.kernel.core.dapper.aspect.JaNodeSpanResolver;
import com.digiwin.dap.nest.kernel.core.dapper.log.DwLog;
import com.digiwin.dap.nest.kernel.core.dapper.meta.NodeKind;
import com.digiwin.dap.nest.kernel.core.dapper.meta.NodeSpan;
import com.digiwin.dap.nest.kernel.core.util.DwStringUtil;
import com.digiwin.dap.nest.kernel.core.util.datastructure.DwCollectionUtil;
import com.digiwin.dap.nest.infrastructure.spring.util.DwSpringContextUtil;
import org.aopalliance.intercept.MethodInvocation;
import org.mybatis.spring.annotation.MapperScan;
import org.mybatis.spring.annotation.MapperScans;
import org.springframework.aop.framework.ReflectiveMethodInvocation;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.stereotype.Component;
import org.springframework.util.ClassUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author chenjian
 * @since 2024年07月08日 13:38:02
 */
@Component
@ConditionalOnClass(name = {"org.mybatis.spring.annotation.MapperScan", DwNodeSpanMybatis.ProxyClassName})
public class DwNodeSpanMybatis implements JaNodeSpanResolver, JaDapperAspectPointcut {

    public static final String ProxyClassName = "org.apache.ibatis.binding.MapperProxy";

    @Override
    public NodeSpan getNodeSpan(MethodInvocation invocation) {
        if (invocation instanceof ReflectiveMethodInvocation) {
            Object proxy = ((ReflectiveMethodInvocation) invocation).getProxy();
            if (proxy.toString().startsWith(ProxyClassName)) {
                return get(NodeKind.Constant.Db);
            }
        }
        return null;
    }

    @Override
    public String getExpression() {
        String expression = DwConfig.get("dapper.aspect.pointcut.mybatis");
        if (DwStringUtil.isNotEmpty(expression)) {
            return expression + " || @within(org.apache.ibatis.annotations.Mapper)";
        }

        List<String> basePackages = new ArrayList<>(addBasePackages(DwSpringContextUtil.getMainApplicationClass()));
//        String[] beanDefinitionNames = JaSpringContextUtil.getApplicationContext().getBeanDefinitionNames();
//        for (String beanDefinitionName : beanDefinitionNames) {
//            basePackages.addAll(addBasePackages(Objects.requireNonNull(JaSpringBeanUtil.getBean(beanDefinitionName)).getClass()));
//        }
        Set<String> basePackageSet = new HashSet<>();
        basePackages.forEach(s -> basePackageSet.addAll(Arrays.asList(s.split(","))));
        if (basePackageSet.size() > 1) {
            DwLog.get().warn("multiple mybatis mapper scan package");
        }

        StringBuilder expressionBuilder = new StringBuilder();
        for (String s : basePackageSet) {
            expressionBuilder.append(String.format("execution(* %s..*.*(..)) || ", s));
        }
        return expressionBuilder + "@within(org.apache.ibatis.annotations.Mapper)";
    }

    private List<String> addBasePackages(Class<?> importingClass) {
        List<String> basePackages = new ArrayList<>();
        MapperScans mapperScans = importingClass.getAnnotation(MapperScans.class);
        if (null != mapperScans) {
            basePackages.addAll(addBasePackages(mapperScans, importingClass));
        } else {
            MapperScan mapperScan = importingClass.getAnnotation(MapperScan.class);
            if (null != mapperScan) {
                basePackages.addAll(addBasePackages(mapperScan, importingClass));
            }
        }
        return basePackages;
    }

    private List<String> addBasePackages(MapperScans mapperScans, Class<?> importingClass) {
        List<String> basePackages = new ArrayList<>();
        if (DwCollectionUtil.isEmpty(mapperScans.value())) {
            basePackages.add(ClassUtils.getPackageName(importingClass.getName()));
        } else {
            for (MapperScan mapperScan : mapperScans.value()) {
                basePackages.addAll(addBasePackages(mapperScan, importingClass));
            }
        }
        return basePackages;
    }


    private List<String> addBasePackages(MapperScan annotation, Class<?> importingClass) {
        List<String> basePackages = new ArrayList<>();
        basePackages.addAll(
                Arrays.stream(annotation.value()).filter(StringUtils::hasText).collect(Collectors.toList()));
        basePackages.addAll(
                Arrays.stream(annotation.basePackages()).filter(StringUtils::hasText).collect(Collectors.toList()));
        basePackages.addAll(
                Arrays.stream(annotation.basePackageClasses()).map(ClassUtils::getPackageName).collect(Collectors.toList()));
        if (DwCollectionUtil.isEmpty(basePackages)) {
            basePackages.add(ClassUtils.getPackageName(importingClass));
        }
        return basePackages;
    }

}
